#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/cdev.h>
#include <linux/device.h>
#include <linux/major.h>
#include <linux/fs.h>
#include <asm/uaccess.h>

#include "adder.h"

static long adder_ioctl(struct file *filp,
			unsigned int cmd,
			unsigned long arg)
{
	void __user *uarg = (void __user *)arg;
	struct adder adder;

	if (copy_from_user(&adder, uarg, sizeof(struct adder))) {
		return -EFAULT;
	}
	adder.r = adder.a + adder.b;
	return copy_to_user(uarg, &adder, sizeof(struct adder));
}

static const struct file_operations adder_fops =
{
	.unlocked_ioctl  = adder_ioctl,
	.owner           = THIS_MODULE,
};

static dev_t dev;
static struct cdev cdev;
static struct class *class;
static struct device *device;

static void __exit adder_exit(void)
{
	device_destroy(class, dev);
	class_destroy(class);
	cdev_del(&cdev);
	unregister_chrdev_region(dev, 1);
}

static int __init adder_init(void)
{
	alloc_chrdev_region(&dev, 0, 1, "adder");
	cdev_init(&cdev, &adder_fops);
	cdev_add(&cdev, dev, 1);
	class = class_create(THIS_MODULE, "adder");
	device = device_create(class, NULL, dev, NULL, "adder");
	return 0;
}

module_init(adder_init);
module_exit(adder_exit);
MODULE_LICENSE("GPL");
